/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


function ImagesPage() // extend WizardPage
{
    var depends = "selected";
    WizardPage.call(this, depends);
}

ImagesPage.prototype =
{
    get filterImages() { return $("cbox_filterImages") },
    
    get pageListbox()
    {
        var deck = $("deck_pages");
        // first panel (index 0) is the no-thumbnail-selected-yet -panel
        return ((deck.selectedIndex == 0) ? null : deck.selectedPanel);
    },
    
    get filter()
    {
        if (! this.filterImages.checked)
            return null;
        
        if (this._filter)
            return this._filter;
        
        var minWidth = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageWidth");
        var minHeight = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageHeight");
        var mode = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageMode");
        var boundingBox = new Box(minWidth, minHeight);
        
        return this._filter = function(rlitem)
        {
            // TODO: it might be better rely image's built-in properties instead of state ("slow" because of event ordering issues, see: _populatePageListbox())
            var image = rlitem.firstChild;
            if ((! image) || (image.state != "loaded"))
                return false;
            var imageBox = new Box(image.naturalWidth, image.naturalHeight);
            return boundingBox.contains(imageBox, (mode == FILTER_MODE_EITHER));
        }
    },
    
    onPageShow: function()
    {
        if (this.firstRun)
        {
            wizard.addPrefObserver(this, "extensions.thumbsdown.scriptwizard.", false);
        }
        
        if (this.dirty.page)
        {
            this.selected = new Array(model.selected.length);
            
            // page loaders
            var self = this;
            var headers = {"Referer": model.referer};
            var timeout = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.requestTimeout");
            
            for each (var thumbnail in model.selected)
            {
                var target = thumbnail.target;
                if (! target.loader)
                {
                    var url = thumbnail.anchor.url;
                    let _thumbnail = thumbnail;
                    var onLoad = function(loader) { self.onPageLoad(loader, _thumbnail); }
                    var onError = function(loader) { self.onPageError(loader, _thumbnail); }
                    target.loader = new ResourceLoader("GET", url, headers, onLoad, onError, timeout);
                    target.loader.load();
                }
            }
            
            // clear left hand side richlistbox
            var left_rlbox = $("rlbox_thumbnails");
            while (left_rlbox.itemCount > 0)
                left_rlbox.removeItemAt(0);
            
            var deck = $("deck_pages");
            deck.selectedIndex = 0;
            
            // clear right hand side deck
            var rlbox = deck.firstChild;
            while (rlbox.nextSibling)
                deck.removeChild(rlbox.nextSibling);
            
            for (var i = 0; i < model.selected.length; i++)
            {
                var thumbnail = model.selected[i];
                var target = thumbnail.target;
                
                // left hand side
                var attrs = {maxwidth: 128, maxheight: 128, src: thumbnail.image.link, class: "show-throbber show-error"};
                var scaledimage = createXulElement("scaledimage", attrs);
                var rlitem = createXulElement("richlistitem", null, [scaledimage]);
                left_rlbox.appendChild(rlitem);
                
                // right hand side
                var rlitem = createXulElement("richlistitem", {class: "page loading"});
                var rlbox = createXulElement("richlistbox", {class: "filteredlistbox", value: thumbnail.index}, [rlitem]);
                if (target.images)
                    this._populatePageListbox(rlbox, target.images);
                
                rlbox.suppressOnSelect = true;
                deck.appendChild(rlbox);
                if (target.image)
                    rlbox.selectedIndex = this.selected[i] = target.image.index;
                else
                    rlbox.selectedIndex = -1;
                rlbox.suppressOnSelect = false;
                
                rlbox.addEventListener("select", function(e) { self.onImageSelect(e); }, false);
            }
            
            this._updateSelectedCount();
            this._updateImageInfo();
            wizard.updateWizardButtons();
        }
    },
    
    observe: function(subject, topic, data)
    {
        switch (topic)
        {
            case "nsPref:changed":
                this.observePreference(subject, topic, data);
                break;
        }
    },
    
    observePreference: function(subject, topic, data)
    {
        var prefNamePrefix = "extensions.thumbsdown.scriptwizard.";
        
        switch (data.substr(prefNamePrefix.length))
        {
            case "minImageWidth":
            case "minImageHeight":
            case "minImageMode":
                // TODO: use non-0 delay
                this._filter = null;
                callLater(0, this, this._updatePageListbox);
                break;
        }
    },
    
    onThumbnailSelect: function()
    {
        this._updateThumbnailInfo();
        this._updatePageListbox();
    },
    
    onPageLoad: function(loader, thumbnail)
    {
        try { this._onPageLoad(loader.content, thumbnail); }
        finally { loader.destroy(); }
    },
    
    onPageError: function(loader, thumbnail)
    {
        try { this._onPageLoad(null, thumbnail); }
        finally { loader.destroy(); }
    },
    
    _onPageLoad: function(doc, thumbnail)
    {
        var target = thumbnail.target;
        
        if (doc)
        {
            var images = [];
            
            for (var i = 0; i < doc.images.length; i++)
            {
                try
                {
                    var img = doc.images[i];
                    image =
                    {
                        alt: img.alt,
                        id: img.id,
                        src: img.src,
                        get link() { return this.src; },
                        url: ioSvc.newURI(img.src, null, null).QueryInterface(Ci.nsIURL),
                        index: images.length
                    }
                    images.push(image);
                }
                catch (e) { /* ignore */ }
            }
            
            target.images = images;
        }
        
        if (page != pages.images)
        {
            this.dirty.page = true;
            return;
        }
        
        // find thumbnail's rlbox
        var deck = $("deck_pages");
        var rlbox = deck.firstChild.nextSibling;
        while (rlbox && (rlbox.getAttribute("value") != thumbnail.index))
            rlbox = rlbox.nextSibling;
        
        this._populatePageListbox(rlbox, target.images);
    },
    
    onImageLoad: function(e)
    {
        var image = e.target;
        var rlitem = image.parentNode;
        var rlbox = rlitem.parentNode;
        var deck = rlbox.parentNode;
        
        if (rlbox != deck.selectedPanel)
            return;
        
        rlbox.applyFilterForItem(rlitem);
        this._updatePageListbox();
    },
    
    onImageSelect: function(e)
    {
        var rlbox = e.target;
        
        // prevent selecting "page" item
        if (rlbox.selectedItem && (rlbox.selectedItem.className.search("page") != -1))
        {
            rlbox.suppressOnSelect = true;
            rlbox.selectedIndex = -1;
            rlbox.suppressOnSelect = false;
            return;
        }
        
        var deck = rlbox.parentNode;
        var thumbnailIndex = deck.selectedIndex - 1;
        this.selected[thumbnailIndex] = ((rlbox.selectedCount != 0) ? parseInt(rlbox.selectedItem.value) : undefined);
        
        this._updateSelectedCount();
        this._updateImageInfo();
        this._updateImageCount();
        wizard.updateWizardButtons();
    },
    
    onFilterImagesChange: function()
    {
        this.pageListbox.applyFilter();
        this._updatePageListbox();
    },
    
    onPageListboxContextMenuShowing: function(e)
    {
        var node = document.popupNode;
        while (node && (node.nodeName != "richlistitem"))
            node = node.parentNode;
        
        if (! node)
        {
            e.preventDefault();
            return;
        }
        
        $("mitem_showLargerImage").disabled = this.filterImages.checked;
    },
    
    onHideSmaller: function()
    {
        var rlbox = this.pageListbox;
        var rlitem = rlbox.selectedItem;
        var image = rlitem.firstChild;
        
        var minWidth = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageWidth");
        var minHeight = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageHeight");
        var mode = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageMode");
        
        var minBox = new Box(minWidth, minHeight);
        var imageBox = new Box(image.naturalWidth, image.naturalHeight);
        minBox.enlarge(imageBox, (mode == FILTER_MODE_EITHER));
        
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minImageWidth", minBox.width);
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minImageHeight", minBox.height);
        
        this.filterImages.checked = true;
        this._updatePageListbox();
    },
    
    onShowLarger: function(e)
    {
        var rlbox = this.pageListbox;
        var rlitem = rlbox.selectedItem;
        var image = rlitem.firstChild;
        
        var minWidth = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageWidth");
        var minHeight = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageHeight");
        var mode = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minImageMode");
        
        var minBox = new Box(minWidth, minHeight);
        var imageBox = new Box(Math.max(0, image.naturalWidth - 1), Math.max(0, image.naturalHeight - 1));
        minBox.shrink(imageBox, (mode == FILTER_MODE_BOTH));
        
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minImageWidth", minBox.width);
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minImageHeight", minBox.height);
        
        this.filterImages.checked = true;
        this._updatePageListbox();
    },
    
    canAdvance: function()
    {
        return (this._count(this.selected) >= 2);
    },
    
    _populatePageListbox: function(rlbox, images)
    {
        if (images && (images.length))
        {
            var self = this;
            
            // remove "page" item
            rlbox.removeChild(rlbox.firstChild);
            
            for (var i = 0; i < images.length; i++)
            {
                var attrs = {maxwidth: 128, maxheight: 128, src: images[i].link, class: "show-throbber show-error"};
                var scaledimage = createXulElement("scaledimage", attrs);
                // load handler is delayed so that scaledimage's own load handler has time to go first
                scaledimage.addEventListener("load", function(e) { callLater(0, self, self.onImageLoad, e); }, false);
                
                var rlitem = createXulElement("richlistitem", {class: "filteredlistitem", value: i}, [scaledimage]);
                
                rlbox.setAttribute("context", "popup_images");
                rlbox.appendChild(rlitem);
            }
        }
        else
        {
            var msg = (images ? _("error.pageHasNoImages") : _("error.pageLoadError"));
            var label = createXulElement("label", {value: msg});
            rlbox.firstChild.appendChild(label);
            rlbox.firstChild.className = "page";  // remove "loading"
            rlbox.disabled = true;
        }
    },
    
    _updateSelectedCount: function()
    {
        $("lbl_images_selectedCount").value = this._count(this.selected) + "/" + this.selected.length;
    },
    
    _updateThumbnailInfo: function()
    {
        var rlbox = $("rlbox_thumbnails");
        var index = rlbox.currentIndex;
        var thumbnail = model.selected[index];
        $("lbl_images_thumbnail").value = thumbnail.image.link;
        $("lbl_images_link").value = thumbnail.anchor.link;
    },
    
    _updatePageListbox: function()
    {
        var rlbox = $("rlbox_thumbnails");
        var index = rlbox.currentIndex;
        $("deck_pages").selectedIndex = index + 1;
        
        this.pageListbox.filter = this.filter;
        
        this._updateSelectedCount();
        this._updateImageInfo();
        this._updateImageCount();
        wizard.updateWizardButtons();
    },
    
    _updateImageInfo: function()
    {
        var rlbox = this.pageListbox;
        var rlitem = (rlbox ? rlbox.selectedItem : null);
        var scaledimage = (rlitem ? rlitem.firstChild : null);
        
        $("lbl_images_image").value = (scaledimage ? scaledimage.src : "");
        if (scaledimage && (scaledimage.state == "loaded"))
        {
            $("lbl_images_imageSize").value = (scaledimage.naturalWidth + "x" + scaledimage.naturalHeight);
            $("lbl_images_imageSize").parentNode.style.visibility = "visible";
        }
        else
            $("lbl_images_imageSize").parentNode.style.visibility = "hidden";
    },
    
    _updateImageCount: function()
    {
        var msg = "";
        
        var rlbox = this.pageListbox;
        if (rlbox)
        {
            var selectedCount = rlbox.selectedCount;
            var rowCount = rlbox.filteredItemCount;
            var imageCount = rlbox.itemCount;
            
            var msg = selectedCount + " / " + rowCount;
            if (rowCount != imageCount)
                msg += " (" + imageCount + ")"
        }
        
        $("lbl_images_imageCount").value = msg;
    },
    
    _count: function(arr)
    {
        return count(arr, function(a) { return (a !== undefined); });
    },
    
    onPageAdvanced: function()
    {
        var selectedImages = [];
        
        for (var i = 0; i < this.selected.length; i++)
        {
            var thumbnail = model.selected[i];
            var imageIndex = this.selected[i];
            if (typeof(imageIndex) == "number")
            {
                thumbnail.target.image = thumbnail.target.images[imageIndex];
                selectedImages.push(thumbnail);
            }
        }
        
        model.images = selectedImages;
        return true;
    }
}

extend(ImagesPage, WizardPage);

pages.images = new ImagesPage();
