/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


function ThumbnailsPage() // extend WizardPage
{
    var depends = "thumbnails";
    WizardPage.call(this, depends);
    
    this.selected = [];
    this.thumbnailIndex = null;
}

ThumbnailsPage.prototype =
{
    get filterThumbnails() { return $("cbox_filterThumbnails") },
    
    get tree() { return $("tree_thumbnails"); },
    
    get filter()
    {
        if (! this.filterThumbnails.checked)
            return null;
        
        var minWidth = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailWidth");
        var minHeight = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailHeight");
        var mode = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailMode");
        var boundingBox = new Box(minWidth, minHeight);
        
        return function(thumbnail)
        {
            var image = thumbnail.image.elem;
            var imageBox = new Box(image.width, image.height);
            return boundingBox.contains(imageBox, (mode == FILTER_MODE_EITHER));
        };
    },
    
    get targetCount()
    {
        if (! this._targetCount)
        {
            var count = {image: 0, page: 0, unknown: 0, error: 0, loading: 0};
            for each (var thumbnail in this.selected)
                if (thumbnail.target.head && thumbnail.target.head.result)
                    count[thumbnail.target.head.result]++;
                else
                    count.loading++
            
            if ((count.image + count.page + count.unknown + count.error + count.loading) != this.selected.length)
                throw new Error("target count mismatch");
            
            this._targetCount = count;
        }
        return this._targetCount;
    },
    
    
    onPageShow: function()
    {
        if (this.firstRun)
        {
            var self = this;
            $("treechildren_thumbnails").addEventListener("DOMMouseScroll", function(e) { self.onThumbnailTreeMouseMove(e); }, false);
            $("image_thumbnail").addEventListener("load", function() { self.onThumbnailLoad(); }, false);
            
            wizard.addObserver(this, TOPIC_TARGET_HEAD, false);
            wizard.addPrefObserver(this, "extensions.thumbsdown.scriptwizard.", false);
        }
        
        if (this.dirty.thumbnails)
            this.tree.view = this.treeView = new ThumbnailsTreeView(model.thumbnails, this.filter);
        
        if (this.dirty.page)
            this._updateSelected();
        
        this._updateThumbnailSize();
    },
    
    observe: function(subject, topic, data)
    {
        switch (topic)
        {
            case "nsPref:changed":
                this.observePreference(subject, topic, data);
                break;
            case TOPIC_TARGET_HEAD:
                this.onTargetHeadLoad(parseInt(data));
                break;
        }
    },
    
    observePreference: function(subject, topic, data)
    {
        var prefNamePrefix = "extensions.thumbsdown.scriptwizard.";
        
        switch (data.substr(prefNamePrefix.length))
        {
            case "minThumbnailWidth":
            case "minThumbnailHeight":
            case "minThumbnailMode":
                // TODO: use non-0 delay
                callLater(0, this, this._updateFilter);
                break;
        }
    },
    
    onTargetHeadLoad: function(index)
    {
        this._targetCount = null;
        
        if (page != pages.thumbnails)
            return;
        
        if (this.treeView.isIndexFiltered(index))
            return;
        
        var row = this.treeView.getIndexRow(index);
        var col = this.tree.columns["tcol_thumbnails_target"];
        this.tree.treeBoxObject.invalidateCell(row, col);
        this._targetCount = null;
        wizard.updateWizardButtons();
    },
    
    onThumbnailTreeSelect: function()
    {
        this._updateSelected();
        wizard.updateWizardButtons();
    },
    
    onThumbnailTreeKeyUp: function()
    {
        this._setThumbnail(this.tree.currentIndex);
    },
    
    onThumbnailTreeMouseMove: function(e)
    {
        var row = this.tree.treeBoxObject.getRowAt(e.clientX, e.clientY);
        this._setThumbnail((row >= 0) ? this.treeView.model[row].index : null);
    },
    
    onThumbnailTreeMouseOut: function(e)
    {
        // TODO: don't remove thumbnail when over context menu
        this._setThumbnail(null);
    },
    
    onThumbnailTreeContextMenuShowing: function(e)
    {
        var row = this.tree.treeBoxObject.getRowAt(e.clientX, e.clientY);
        if (row == -1)
        {
            e.preventDefault();
            return;
        }
        
        $("mitem_showLargerThumbnail").disabled = this.filterThumbnails.checked;
    },
    
    onThumbnailLoad: function()
    {
        if (page != pages.thumbnails)
            return;
        
        this._updateThumbnailSize();
    },
    
    onFilterThumbnailsChange: function()
    {
        this._updateFilter();
    },
    
    onHideSmaller: function()
    {
        var row = this.tree.currentIndex;
        var thumbnail = this.treeView.model[row];
        var image = thumbnail.image.elem;
        
        var minWidth = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailWidth");
        var minHeight = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailHeight");
        var mode = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailMode");
        
        var minBox = new Box(minWidth, minHeight);
        var imageBox = new Box(image.width, image.height);
        minBox.enlarge(imageBox, (mode == FILTER_MODE_EITHER));
        
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minThumbnailWidth", minBox.width);
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minThumbnailHeight", minBox.height);
        
        this.filterThumbnails.checked = true;
        this._updateFilter();
    },
    
    onShowLarger: function()
    {
        var row = this.tree.currentIndex;
        var thumbnail = this.treeView.model[row];
        var image = thumbnail.image.elem;
        
        var minWidth = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailWidth");
        var minHeight = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailHeight");
        var mode = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.minThumbnailMode");
        
        var minBox = new Box(minWidth, minHeight);
        var imageBox = new Box(Math.max(0, image.width - 1), Math.max(0, image.height - 1));
        minBox.shrink(imageBox, (mode == FILTER_MODE_BOTH));
        
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minThumbnailWidth", minBox.width);
        nsPreferences.setIntPref("extensions.thumbsdown.scriptwizard.minThumbnailHeight", minBox.height);
        
        this.filterThumbnails.checked = true;
        this._updateFilter();
    },
    
    canAdvance: function()
    {
        if (this.selected.length < 2)
            return false;
        if ((this.targetCount.image == this.selected.length) ||
            (this.targetCount.page == this.selected.length))
            return true;
        return false;
    },
    
    _setThumbnail: function(index)
    {
        if (index == this.thumbnailIndex)
            return;
        
        this.thumbnailIndex = index;
        
        var image = $("image_thumbnail");
        if (index != null)
        {
            var thumbnail = model.thumbnails[index];
            
            var boxComputedStyle = document.defaultView.getComputedStyle($("box_thumbnail"), null);
            var boxWidth = boxComputedStyle.width.replace(/px$/, "");
            var boxHeight = boxComputedStyle.height.replace(/px$/, "");
            
            image.src = thumbnail.image.link;
            image.setSize(boxWidth, boxHeight);
        }
        else
        {
            image.src = "";
            this._updateThumbnailSize();
        }
    },
    
    _updateSelected: function()
    {
        this.selected = [this.treeView.model[row] for each (row in getTreeSelectionIterator(this.tree.view.selection))];
        this._targetCount = null;
        
        var selectedCount = this.selected.length;
        var rowCount = this.treeView.rowCount;
        var thumbnailCount = this.treeView.originalCount;
        
        var msg = selectedCount + " / " + rowCount;
        if (rowCount != thumbnailCount)
            msg += " (" + thumbnailCount + ")"
        
        $("lbl_thumbnails_count").value = msg;
    },
    
    _updateThumbnailSize: function()
    {
        var index = this.thumbnailIndex;
        var show = (index != null);
        if (show)
        {
            var image = model.thumbnails[index].image.elem;
            $("lbl_thumbnailSize").value = image.width + "x" + image.height;
        }
        $("lbl_thumbnailSize").parentNode.style.visibility = (show ? "visible" : "hidden");
    },
    
    _updateFilter: function()
    {
        this.treeView.filter = this.filter;
        
        if (page != pages.thumbnails)
        {
            this.dirty.page = true;
            return;
        }
        
        this._updateSelected();
    },
    
    onPageAdvanced: function()
    {
        model.selected = this.selected;
        
        if (! model.matchingScript)
            model.matchingScript = scriptWizard.getMatchingScript(model.selected);
        
        if (model.matchingScript)
            return wizard.advance("script");
        
        if (! model.linkUrlTokens)
            try
            {
                model.linkUrlTokens = scriptWizard.getLinkUrlTokens(model.selected);
                model.error = null;
            }
            catch (e if (typeof(e) == "string"))
            {
                model.error = e;
                return wizard.advance("error");
            }
        
        if (! model.target)
            model.target = ((this.targetCount.image > 0) ? TARGET_IMAGE : TARGET_PAGE);
        
        switch (model.target)
        {
            case TARGET_IMAGE: return wizard.advance("mode");
            case TARGET_PAGE: return wizard.advance("images");
        }
        
        throw new Error("unreachable code");
    }
}

extend(ThumbnailsPage, WizardPage);

pages.thumbnails = new ThumbnailsPage();


function ThumbnailsTreeView(thumbnails, filter) // extend FilteredTreeView
{
    FilteredTreeView.call(this, thumbnails, filter);
}

ThumbnailsTreeView.prototype =
{
    getCellText: function(row, col)
    {
        var thumbnail = this.model[row];
        switch (col.id)
        {
            case "tcol_thumbnails_link":
                return thumbnail.anchor.link;
            
            case "tcol_thumbnails_target":
                var target = thumbnail.target.head;
                if (! (target && target.result))
                    return _("target.loading");
                switch (target.result)
                {
                    case "image":   return _("target.image");
                    case "page":    return _("target.page");
                    case "unknown": return (target.contentType ? _("target.unknown", (target.mimeType ? target.mimeType : target.contentType)) : _("target.noContentType"));
                    case "error":   return _("target.error", (target.status ? target.status : -1));
                    case "timeout": return _("target.timeout");
                }
                throw new Error("unknown target result: " + target.result);
        }
        throw new Error("unknown column: " + col.id);
    }
}

extend(ThumbnailsTreeView, FilteredTreeView);
